#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look
#  using RobA's BORDER INWARD REPEAT + top-over G'MIC effects

# (c) Dinasset 2020
#
#   History:
#
#   v1.0: 2021-01-31: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from gimpfu import *
import decimal
import random
import sys, os
import time
import datetime

# define here some constant value, when required
FixWH=3600


############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

# ========================================================
#   ANCILLARY MODULE : BORDER INWARD REPEAT (from RobA)
# --------------------------------------------------------
def BordersInwardRepeat (img, inLayer, inSpacing, inBSize, inColour) :
    
    width = img.width 
    height = img.height
    
    colourtoggle = 1          
     
    initial_selection = 0

    # save selection or select all if no selection
    if pdb.gimp_selection_is_empty (img) == False:
        initial_selection = pdb.gimp_selection_save (img)
    else:
        pdb.gimp_selection_all (img) 

    scratch_layer = inLayer.copy()
    img.add_layer (scratch_layer, -1)
    # fill the scratch layer with white
    scratch_layer.fill (WHITE_FILL)
    
    # work in scratch layer
    pdb.gimp_selection_shrink (img, int(inSpacing/2))  # 202009
    while pdb.gimp_selection_is_empty (img) == False:
        if colourtoggle == 0:
            gimp.set_foreground ((255, 255, 255, 255))
            colourtoggle = 1
        else:
            gimp.set_foreground ((0, 0, 0, 255))
            colourtoggle = 0
        pdb.gimp_edit_fill (scratch_layer, FOREGROUND_FILL)     
        pdb.gimp_selection_shrink (img, inSpacing)

    # select black
    pdb.gimp_image_select_color (img, CHANNEL_OP_REPLACE, scratch_layer, ((0, 0, 0, 255)))
    
    # turn it into a path
    
    if pdb.gimp_selection_is_empty (img) == False:  # added on 21.1
        pdb.plug_in_sel2path (img, scratch_layer)
        paths = img.vectors
        newpath = paths[0] 

        # make it visible and active
        newpath.visible = True
        pdb.gimp_image_set_active_vectors (img, newpath)
        
    # delete scratch layer
    img.remove_layer (scratch_layer)
    
    # load initial selection back up and delete the channel
    if initial_selection <> 0:
        pdb.gimp_selection_load (initial_selection)
        img.remove_channel (initial_selection)
    else:
        pdb.gimp_selection_none (img) # else clear the selection

    if pdb.gimp_selection_is_empty (img) == False:  # added on 21.1
        gimp.set_foreground (inColour)
        pdb.gimp_context_set_brush ("2. Hardness 100")
        pdb.gimp_context_set_brush_angle (0.00)
        pdb.gimp_context_set_brush_aspect_ratio (0.00)
        pdb.gimp_context_set_brush_size (inBSize)
        pdb.gimp_context_set_opacity (100.0)
        pdb.gimp_context_set_dynamics ("Dynamics Off")
        pdb.gimp_context_set_brush_force (1.0)
    
        pdb.gimp_drawable_edit_stroke_item (inLayer, newpath)

        pdb.gimp_image_remove_vectors (img, newpath)
 
    #done =======================================================

    return (inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    mofified=False
    if meanl < 110:
        #info_msg("mean value of pattern = "+str(meanl))
        if meanl < 80: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.3,0.6, 0.95,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        while meanl < 110:
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.05, 0.00, 1.00, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        mofified=True
        #info_msg("modified mean luminance = "+str(meanl))
    if meanl > 144:
        #info_msg("mean value found = "+str(meanl))
        if meanl > 208: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.5,0.375, 1.0,0.75))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        if meanl > 176: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.5,0.45, 1.0,0.90))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        while meanl > 144:
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.95, 0.00, 1.00, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        mofified=True
        #info_msg("modified mean value = "+str(meanl))
    
    return(inLayer)
     
############################################################################
#     M   A  I  N
#---------------------------------------------------------------------------

def DrawStyleM (inImage, inLayer,
                    widthheight,
                    nrAreas,
                    which_outcome,
                    which_bg,
                    doFinValAdj,
                    Mask,
                    doFlatten
                   ) :
 
    pdb.gimp_context_push()
    
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    name = inLayer.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ", as a drawing using RobA's BORDER INWARD REPEAT on "
    suffix = suffix+str(int(nrAreas))+" area colours"
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # calculate scale factor for the new output image
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight                                    
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
     
    if pdb.gimp_selection_is_empty (inImage) == True:
        workingon = "fulldrawable"
    else :
        workingon = "selection"
        original_selection = pdb.gimp_selection_save (inImage)
        
    currentBG = pdb.gimp_context_get_background()
    currentFG = pdb.gimp_context_get_foreground()
    currentPAT = pdb.gimp_context_get_pattern()

# --------------------------------------------------------------------------------------
    newImage = pdb.gimp_image_duplicate (inImage)                    
    newBG = newImage.layers[0]     # background layer
    #newImage.add_layer (newBG, 0)
    pdb.gimp_layer_add_alpha (newBG)              #  RGBA
    newBG.name = "newBG: BG of the newly created image"
    newBG.mode = LAYER_MODE_NORMAL
    newBG.opacity = 100.0 
    
    #pdb.gimp_display_new (newImage)
    #return
    
    underBG = newBG.copy()
    newImage.add_layer (underBG, 1)               #  to keep the non selected area of the source layer below
    pdb.gimp_layer_add_alpha (underBG)            #  RGBA
    underBG.name = "underBG: the original BG under the BG for the selection"
    newBG.mode = LAYER_MODE_NORMAL
    newBG.opacity = 100.0 
    
    #pdb.gimp_display_new (newImage)
    #return
    
    original = newBG.copy()
    newImage.add_layer (original, 2)              #  to keep the source layer AS IS below
    pdb.gimp_layer_add_alpha (original)           #  RGBA
    original.name = "original: the original BG will be shown under the flattened outcome"
    newBG.mode = LAYER_MODE_NORMAL
    newBG.opacity = 100.0 
    pdb.gimp_item_set_visible (original, False)
    
    pdb.gimp_image_scale (newImage, rwidth, rheight)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    pdb.gimp_image_undo_freeze (newImage)                           # save time and space
    
    pdb.gimp_selection_invert (newImage)
    
    if workingon == "selection" :
        pdb.gimp_selection_invert (newImage)
        pdb.gimp_edit_cut (underBG)                                 # keep ONLY the non selected area of the source layer
        pdb.gimp_selection_invert (newImage)
        pdb.gimp_edit_cut (newBG)
        pdb.gimp_selection_invert (newImage)
        
        # save selection   # shifted on 6.1.21
        saved_selection = pdb.gimp_selection_save (newImage)
        pdb.gimp_selection_none (newImage)
        pdb.gimp_image_set_active_layer (newImage, newBG)
    
    #pdb.gimp_display_new (newImage)
    #return
    
#####################################################################################################
#   quantize the colour_areas using mode indexed on a duplicated image (WORK)
# =================================================================================================== 

    WORK_IMAGE = pdb.gimp_image_new (rwidth, rheight, RGB)  # do not use duplicate, indexed wrong!
    WORK_DRAWABLE = pdb.gimp_layer_new_from_drawable (newBG, WORK_IMAGE)
    WORK_IMAGE.add_layer (WORK_DRAWABLE, 0)
    WORK_DRAWABLE.name = "WORK_DRAWABLE. BG Layer"
    
    pdb.plug_in_sel_gauss (WORK_IMAGE, WORK_DRAWABLE, FixWH/200, 50)
    
    pdb.gimp_drawable_curves_spline (WORK_DRAWABLE, HISTOGRAM_VALUE,    # only medium values
                                     6, (0.00,0.25,0.50,0.50,1.00,0.75))
    
    pdb.gimp_image_convert_indexed (WORK_IMAGE, 0, 0, nrAreas, 0, 0, "ignored")    # INPUT   <<<<<<<< 
    # get the number of pixels in the colormap (which exists for indexed images)
    (count, colors) = pdb.gimp_image_get_colormap (WORK_IMAGE)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas = (count/3) 
    areas_palette = pdb.gimp_palette_new(pdb.gimp_image_get_name (WORK_IMAGE))
    
    index = 0
    while index < count:
    # Compute the color
        color = (colors[index], colors[index+1], colors[index+2])
    # Add to the filter temporary palette
        pdb.gimp_palette_add_entry (areas_palette, str(index/3), color)
        index+=3
    # the temporary palette has been created            

    # re-set image to RGB to proceed
    pdb.gimp_image_convert_rgb (WORK_IMAGE)
    pdb.gimp_layer_add_alpha (WORK_DRAWABLE)                        #  RGBA
    bsize = max(2, round (widthheight*1.5/3600.0, 2))
    #info_msg ("brush size = "+str(bsize))
    spread = bsize
     
#   =========================================
#   loop to get the meanvalues of the N areas                 
#   =========================================

    areaNR = 0
    minmean = 255
    maxmean = 0
    summean = 0
    areas = 0
    means = []
    for areaNR in range (0, nr_areas):
        one_color = pdb.gimp_palette_entry_get_color (areas_palette, areaNR)
        pdb.gimp_image_select_color (WORK_IMAGE, CHANNEL_OP_REPLACE, WORK_DRAWABLE, one_color)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds (WORK_IMAGE)
        if there_is_a_selection == 1:
            (mean, stddev, median, pixels, count, percent) = pdb.gimp_histogram (WORK_DRAWABLE, 0, 0, 255)
            means.append (int(round(mean)))
            summean += int(round(mean))
            if int(round(mean)) < minmean:
                minmean = int(round(mean))
            if int(round(mean)) > maxmean:
                maxmean = int(round(mean))
            areas += 1
    #info_msg ("end loop")
    meansasc = sorted(means, key=int)               # ascending order
    #info_msg (str(meansasc))

#   ==================================    
#   loop to DRAW the N areas for tiles
#   part 1: borders inward
#   ================================== 

    pdb.gimp_image_undo_freeze (WORK_IMAGE)                      # save time and space
    
    WORK_TILES = WORK_DRAWABLE.copy()
    WORK_IMAGE.add_layer (WORK_TILES, 0)
    WORK_TILES.name = "WORK_TILES. Layer for Tiles (RobA)"
    
    pdb.gimp_selection_all (WORK_IMAGE) # 6.1.21
    (mn,sd,me,px,totcount,pc) = pdb.gimp_drawable_histogram (WORK_TILES,HISTOGRAM_VALUE,
                                                             0.0, 1.0) # 6.1.21
    #info_msg ("totcount="+str(int(totcount)))
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
    areaNR = 0
    area_color = []   # empty list
    pdb.gimp_context_set_sample_threshold (0.01)

    for areaNR in range (0, nr_areas):
        one_color = pdb.gimp_palette_entry_get_color (areas_palette, areaNR)
        area_color.append (one_color)
        (Ra,Ga,Ba,Aa) = one_color 

        pdb.gimp_image_select_color (WORK_IMAGE, CHANNEL_OP_REPLACE, WORK_TILES, one_color)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds (WORK_IMAGE)
        if there_is_a_selection == 1:
             
            color_area = pdb.gimp_image_get_selection (WORK_IMAGE)
            this_channel = pdb.gimp_selection_save (WORK_IMAGE)
             
            (mean,sd,me,px,cn,pc) = pdb.gimp_drawable_histogram (WORK_TILES,HISTOGRAM_VALUE,
                                                                 0.0, 1.0)
            areaquota=(int(cn*100)/int(totcount))
            #info_msg ("cn="+str(int(cn))+" areaquota="+str(areaquota)+
            #          " area="+str(areaNR+1)+" color="+str(one_color))   # 6.1.21
            
            irmean=int(round(mean))
            #info_msg ("irmean="+str(irmean))
            spacing=bsize*6  
            #info_msg ("area nr "+str(areaNR)+" mean = "+str(irmean)+
            #          " spacing = "+str(spacing)+" bsize = "+str(bsize))   
            pdb.gimp_context_set_brush ("2. Hardness 100")
            pdb.gimp_context_set_brush ("2. Star")  # experiment
            pdb.gimp_context_set_brush_size (bsize)       
            if irmean>127.5:
                strokeval = max(63,irmean-63)   #min
            else:
                strokeval = min(192,irmean+63)  #max
            # do not stroke small areas !
            if areaquota>3: # only greater than 3 percent   # 6.1.21
                BordersInwardRepeat (WORK_IMAGE, WORK_TILES, spacing, bsize,
                                 ((strokeval,strokeval,strokeval,255))) # was bsize*1.5
        else:
            info_msg ("area #"+str(int(areaNR+1))+" no selection")
    pdb.gimp_selection_none (WORK_IMAGE)
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
    #info_msg ("end loop")
    pdb.plug_in_spread (WORK_IMAGE, WORK_TILES, spread*2, spread*2)
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
    pdb.gimp_drawable_levels (WORK_TILES, HISTOGRAM_VALUE, 0.25, 0.75, False, 
                                      1.00, 0.00, 1.00, False)  # add contrast
    pdb.gimp_drawable_levels_stretch (WORK_TILES)   # use instead of repeating
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
#   ================================   
#   DRAW the countours of the areas
#   ================================ 
     
    WORK_CONTOURS = WORK_DRAWABLE.copy()    
    WORK_IMAGE.add_layer (WORK_CONTOURS, 0)
    pdb.gimp_item_set_visible (original, False) # reset
    pdb.gimp_layer_flatten (WORK_CONTOURS)     
    WORK_CONTOURS.name = "WORK_CONTOURS. Layer for Contours"
    WORK_CONTOURS.mode = LAYER_MODE_NORMAL
    pdb.plug_in_edge (WORK_IMAGE, WORK_CONTOURS, 1.0, 0, 0) # NONE, SOBEL
    pdb.gimp_threshold (WORK_CONTOURS, 15, 255)
    pdb.plug_in_vinvert (WORK_IMAGE, WORK_CONTOURS)
    #pdb.plug_in_erode (WORK_IMAGE, WORK_CONTOURS, 1, HISTOGRAM_VALUE, 0.5, 7, 0, 255)
    pdb.plug_in_colortoalpha (WORK_IMAGE, WORK_CONTOURS, (255,255,255,255))
    
    #pdb.gimp_drawable_levels (WORK_CONTOURS, HISTOGRAM_VALUE, 0.0, 1.0, False,
    #                                  1.00, 0.40, 1.00, False)  # light borders
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
#   ============================= 
#   merge to create the "texture"
#   =============================

    pdb.gimp_item_set_visible (WORK_DRAWABLE, False)
    
    WORK_MERGED = pdb.gimp_image_merge_down (WORK_IMAGE, WORK_CONTOURS, CLIP_TO_BOTTOM_LAYER)
    WORK_MERGED.name = "Merged Layers Contours and Tiles"
    
    #pdb.plug_in_emboss (WORK_IMAGE, WORK_MERGED, 60.0, 45.0, 20.0, 0)
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return
    
#================================================================================================ 
#   COPY THE OUTCOME OF THE PROCESSED WORKING IMAGE TO THE OUTPUT IMAGE    
# =============================================================================================== 
    
    BorderedLayer = pdb.gimp_layer_new_from_drawable (WORK_MERGED, newImage)
    pdb.gimp_image_delete (WORK_IMAGE)          # <<<<<<<<<<<<<<<<<<<<<<<<< not needed any more
    BorderedLayer.name =  "BorderedLayer. Intermediate Outcome"
    newImage.add_layer (BorderedLayer, 0)
    BorderedLayer.mode =  LAYER_MODE_NORMAL
    BorderedLayer2 = BorderedLayer.copy()
    newImage.add_layer (BorderedLayer2, 0)
    BorderedLayer2.name =  "BorderedLayer. Intermediate Outcome -inverted"
    pdb.plug_in_vinvert (newImage, BorderedLayer2)
    pdb.plug_in_gauss (newImage, BorderedLayer2, spread*5, spread*5, 1)
    pdb.gimp_drawable_desaturate (BorderedLayer2, DESATURATE_LUMA)
    BorderedLayer2.mode =  LAYER_MODE_GRAIN_MERGE   #SOFTLIGHT
    BorderedLayer2.opacity = 50.0
    
    #pdb.gimp_display_new (newImage)
    #return

    # ===================
    # which final outcome
    # -------------------
    PreFinalL = pdb.gimp_layer_new_from_visible (newImage, newImage, "PreFinal Layer ")
    newImage.add_layer (PreFinalL, 0)
    PreFinalL.opacity = 100.0
    PreFinalL.mode = LAYER_MODE_NORMAL
    
    OUTTYPE=["0 simple",
             "1 whirls",
             "2 turbolent halftone",
             "3 seamless turbolence",
             ]
    #info_msg ("which_outcome="+str(OUTTYPE[which_outcome]))
    TopEffect = PreFinalL.copy()
    newImage.add_layer (TopEffect, 0)
    pdb.gimp_drawable_curves_spline (TopEffect, HISTOGRAM_VALUE,    # only medium values
                                     6, (0.00,0.25,0.50,0.50,1.00,0.75))
    #TopEffect.mode = LAYER_MODE_GRAIN_MERGE
    TopEffect.opacity = 100.0
    
    #pdb.gimp_display_new (newImage)
    #return
    
    if which_outcome == 0 :     # none
        suffix = suffix + " (simple)"
        TopEffect.name = "itself"
    if which_outcome == 1 :     # whirls
        suffix = suffix + " (G'mic whirls)"
        #[G'MIC] Whirls: fx_whirls 7,2,0.2,1.8,11,0,50,50   # REPL BY BELOW
        #[G'MIC] Whirl Drawing: fx_draw_whirl 20,0,50,50    # new
        pdb.plug_in_gmic_qt(newImage, TopEffect, 1,0 ,
        #   "-v - -fx_whirls 7,2,0.2,1.8,11,0,50,50 ")      # repl by below
            "-v - -fx_draw_whirl 20,0,50,50 ")              # new
        TopEffect = newImage.layers[0]
        TopEffect.name = "[GMIC] -fx_whirls 7,2,0.2,1.8,11,0,50,50 "
    if which_outcome == 2 :     # turbo halftone
        suffix = suffix + " (G'mic turbo halftone)"
        #[G'MIC] Turbulent Halftone: iain_turbulent_halftone 5,40,0,1,512,0.75,0,0,0
        pdb.plug_in_gmic_qt(newImage, TopEffect, 1,0 ,
            "-v - -iain_turbulent_halftone 5,40,0,1,512,0.75,0,0,0 ")
        TopEffect = newImage.layers[0]
        TopEffect.name = "[GMIC] -iain_turbulent_halftone 4,40,0,1,512,0.75,0,0,0 "
    if which_outcome == 3 :     # seamless turbo
        suffix = suffix + " (G'mic seamless turbo)"
        #[G'MIC] [G'MIC] Seamless Turbulence: fx_seamless_turbulence 5,20,0,1,3,1
        pdb.plug_in_gmic_qt(newImage, TopEffect, 1,0 ,
            "-v - -fx_seamless_turbulence 5,20,0,1,3,1 ")
        TopEffect = newImage.layers[0]
        TopEffect.name = "[GMIC] -fx_seamless_turbulence 5,20,0,1,3,1 "
        
    TopEffect.mode = LAYER_MODE_GRAIN_MERGE
    
    # -----------------------------------------------------------------------------------------------
    
    #pdb.gimp_display_new (newImage)
    #return
    
    pdb.gimp_context_pop    # restore  
    # add the requested BG
    if which_bg==0: # *none
        reqdBG = PreFinalL.copy()
        newImage.add_layer (reqdBG, 1)
        reqdBG.name = "BG=*none"
    if which_bg==1: # source image
        # additions to include the original
        pdb.gimp_item_set_visible (original, True) 
        reqdBG = original.copy()
        newImage.add_layer (reqdBG, 1)
        reqdBG.name = "BG=original"
        pdb.gimp_item_set_visible (original, False)
    if which_bg==2: # active BG
        reqdBG = PreFinalL.copy()
        newImage.add_layer (reqdBG, 1)
        pdb.gimp_context_set_background (currentBG)
        pdb.gimp_drawable_edit_fill (reqdBG, FILL_BACKGROUND)
        reqdBG.name = "BG=active BG"
    if which_bg==3: # active FG
        reqdBG = PreFinalL.copy()
        newImage.add_layer (reqdBG, 1)
        pdb.gimp_context_set_foreground (currentFG)
        pdb.gimp_drawable_edit_fill (reqdBG, FILL_FOREGROUND)
        reqdBG.name = "BG=active FG"
    if which_bg==4: # active Pattern
        reqdBG = PreFinalL.copy()
        newImage.add_layer (reqdBG, 1)
        pdb.gimp_context_set_pattern (currentPAT)
        pdb.gimp_drawable_edit_fill (reqdBG, FILL_PATTERN)
        reqdBG.name = "BG=active pattern"
    reqdBG.mode = LAYER_MODE_GRAIN_MERGE
    reqdBG.opacity = 100.0
    suffix=suffix+","+reqdBG.name
    
    if Mask>0:
        maskedBG = newBG.copy()
        newImage.add_layer (maskedBG, 0)
        maskedBG.name = "BG on Top for portraits touch-up "

        maskedBG.mode = LAYER_MODE_NORMAL                                 
        maskedBG.opacity = 90.0                                             
        #                                                                     
        if Mask==1:
            blackmask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_BLACK)    
            pdb.gimp_layer_add_mask (maskedBG, blackmask)
            suffix=suffix+" (user mask)"
        if Mask==2:
            graymask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_COPY)      
            pdb.gimp_layer_add_mask (maskedBG, graymask)
            suffix=suffix+" (gray copy mask)"
        pdb.gimp_item_set_visible (maskedBG, False)
        
    if doFlatten ==True:
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
    else:    
        Outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, " Outcome ")
        newImage.add_layer (Outcome, 0)
    Outcome.opacity = 100.0
    Outcome.mode = LAYER_MODE_NORMAL
    Outcome.name = st+" "+name+suffix 
    if doFlatten == False:
        pdb.gimp_item_set_visible (Outcome, False)
    if doFinValAdj == True:
        Outcome = adjustmeanv (newImage, Outcome)
    if Mask>0:
        pdb.gimp_item_set_visible (maskedBG, True)
    if workingon == "fulldrawable" :
        pdb.gimp_selection_none (inImage)
        pdb.gimp_selection_none (newImage)
    else:
        pdb.gimp_image_select_item (inImage, CHANNEL_OP_REPLACE, original_selection)
        pdb.gimp_image_select_item (newImage, CHANNEL_OP_REPLACE, saved_selection)
        pdb.gimp_selection_invert (newImage)
        pdb.gimp_edit_cut (Outcome)
        if doFlatten==False:
            pdb.gimp_edit_cut (TopEffect)
            pdb.gimp_edit_cut (reqdBG)
        pdb.gimp_selection_invert (newImage)
        
    pdb.gimp_display_new (newImage)    
    #pdb.gimp_displays_flush
    pdb.gimp_palette_delete(areas_palette)
    
    return(Outcome)

# =====================================================================================================
     
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2020"
desc="Draw Effect Style M "

register(
    "DrawStyleM",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+"...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)",+
                               3600, (0, 9600, 300)),
      (PF_ADJUSTMENT, "areas",  "Poster Areas", 12, (3, 15, 1)),
      (PF_OPTION, "color", "Top G'MIC effect", 2, ("*none",
                                                   "whirls",
                                                   "turbolent halftone",
                                                   "seamless turbolence",
                                                      )),
      (PF_OPTION, "bgr", "Background option", 1, ("*none",
                                                  "Source image",
                                                  "Active BG",
                                                  "Active FG",
                                                  "Active Pattern",
                                                  )),
      (PF_TOGGLE, "adjfinval", "Do adjust final value?", True),
      (PF_OPTION, "maskbg", "Add Masked BG on Top?"+
                    "\n  (for final adjustments)", 2, ["*none",
                                                       "black (user paints over)",
                                                       "gray copy"]),
      (PF_TOGGLE, "flatten", "Do flatten?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    DrawStyleM,
    menu=menu
    )

main()